﻿using System;
using System.Collections.Generic;
using Pz.API.QH360.SDK.Common;
using Pz.API.QH360.SDK.Enum;
using Pz.API.QH360.SDK.Model;
namespace Pz.API.QH360.SDK.API
{
    /// <summary>
    /// 账户服务
    /// </summary>
    public class APIAccount : APIParent
    {
        /// <summary>
        /// 获取服务的类型
        /// </summary>
        internal override ServiceType ServiceType
        {
            get { return ServiceType.account; }
        }

        #region 服务接口类
        /// <summary>
        /// 获取推广计划id列表
        /// </summary>
        /// <param name="campaignIds">计划id列表数组</param>
        /// <returns>Result.</returns>
        public Result getCampaignIdList(ref long[] campaignIds)
        {
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.methodType = MethodType.getCampaignIdList;
            req.postResponse(null);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MCampaignIdList m = Json.Deserialize<MCampaignIdList>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    campaignIds = m.campaignIdList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 推广帐号的每日预算设置
        /// </summary>
        /// <param name="price">预算值(设置值必须大于等于10，且必须为10的整倍数或等于0时认为不设置预算。)</param>
        /// <returns>Result.</returns>
        public Result updateBudget(int price)
        {
            //确保账户的预算值为10的整倍数
            int r = price % 10;
            if (r != 0)
                price = price / 10 * 10;
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.methodType = MethodType.updateBudget;
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("budget", price));
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    AffectedRecords apiRes = Json.Deserialize<AffectedRecords>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 获取推广帐号下的完整数据或指定计划的完整数据，帐户数据文件下载为异步请求，需要根据接
        ///口返回的fileId查询文件生成状态和下载文件
        /// </summary>
        /// <param name="idList">json格式的推广计划的id数组，如[234,799]，不传为null默认全帐号下的数据</param>
        /// <param name="fileId">返回的fileId</param>
        /// <returns>Result.</returns>
        public Result getAllObjects(long[] idList, ref string fileId)
        {
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.methodType = MethodType.getAllObjects;
            List<MyPostParameter> pars = null;
            if (idList != null && idList.Length > 0)
            {
                pars = new List<MyPostParameter>();
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            }
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    AccountFile m = Json.Deserialize<AccountFile>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    fileId = m.fileId;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
        /// <summary>
        /// 在调用getAllObjects接口后使用，查询数据文件是否已经生成 
        /// </summary>
        /// <param name="fileId">生成文件的fileId</param>
        /// <param name="generated">数据文件的当前状态，pending表示等待中，process表示正在生成中，success表示已经成功生成，fail表示生成失败</param>
        /// <param name="filePath">isGenerated为success时包含该字段，为下载文件的UR</param>
        /// <param name="fileMd5">isGenerated 为success时包含该字段，为数据文件的MD5值</param>
        /// <returns>Result.</returns>
        public Result getFileState(string fileId, ref Generated generated, ref string filePath, ref string fileMd5, ref long fileSize)
        {
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.methodType = MethodType.getFileState;
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("fileId", fileId));
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    FileStateInfo m = Json.Deserialize<FileStateInfo>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    generated = (Generated)System.Enum.Parse(typeof(Generated), m.isGenerated);
                    filePath = m.filePath;
                    fileMd5 = m.fileMd5;
                    fileSize = m.fileSize;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 获取账户信息
        /// </summary>
        /// <param name="accountInfo">账户信息实体
        ///uid  账户的id 
        ///userName  账户名 
        ///balance  余额 
        ///Budget  预算 
        ///Resources /投放资源，当前只有一个值：“search”，代表当前账户可以进行搜索推广 
        ///allowDomain  允许推广的域名，多个域名之间半角逗号分隔  </param>
        /// <returns>Result.</returns>
        public Result getInfo(ref MAccountInfo accountInfo)
        {
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.MaxExceptionCount = 1;
            req.webTimeOut = MyConfig.WebTimeOut;
            req.methodType = MethodType.getInfo;
            req.postResponse(null);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MAccountInfo m = Json.Deserialize<MAccountInfo>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    accountInfo = m;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 获取账户排除ip
        /// </summary>
        /// <param name="item">Json格式化后的排除ip列表字符串,最后一列可能是 * 号</param>
        /// <returns>Result.</returns>
        public Result getExcludeIp(ref string ipListJson)
        {
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.MaxExceptionCount = 1;
            req.webTimeOut = MyConfig.WebTimeOut;
            req.methodType = MethodType.getExcludeIp;
            req.postResponse(null);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    ExcludeIpList m = Json.Deserialize<ExcludeIpList>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    if (m.excludeIpList != null)
                        ipListJson = Json.SerializeObject(m.excludeIpList);
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
        /// <summary>
        /// 更新IP排除列表
        /// </summary>
        /// <param name="item">Json格式化后的ip数组，正确的ip 或最后一位为*号，最多10个，每次更新均为全量覆盖方式。 </param>
        /// <returns>Result.</returns>
        public Result updateExcludeIp(string ipListJson)
        {
            if (string.IsNullOrEmpty(ipListJson) || "[]" == ipListJson)
                ipListJson = "[]";
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.methodType = MethodType.updateExcludeIp;
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("excludeIpList", ipListJson));
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    ApiResult apiRes = Json.Deserialize<ApiResult>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 客户登陆
        /// </summary>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public Result clientLogin(string username, string password, ref LoginToken loginToken)
        {
            string encrypPassword = SymmetricMethod.MyEncryption(password,AuthHeader.apiSecretID);
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.MaxExceptionCount = 1;
            req.webTimeOut = MyConfig.WebTimeOut;
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("username", username));
            pars.Add(new MyPostParameter("passwd", encrypPassword));
            string url = string.Format(MyConfig.APIHost + "/{0}/{1}", ServiceType.ToString(), MethodType.clientLogin);
            req.postResponse(pars,url);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    loginToken = Json.Deserialize<LoginToken>(req.data);
                    if (null == loginToken)
                        break;
                    res.Failures = loginToken.failures;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 客服登陆
        /// </summary>
        /// <param name="username"></param>
        /// <param name="password"></param>
        /// <returns></returns>
        public Result managerLogin(string username, string password, ref List<LoginToken> managerTokenList)
        {
            string encrypPassword = SymmetricMethod.MyEncryption(password, AuthHeader.apiSecretID);
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.MaxExceptionCount = 1;
            req.webTimeOut = MyConfig.WebTimeOut;
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("username", username));
            pars.Add(new MyPostParameter("passwd", encrypPassword));
            string url = string.Format(MyConfig.APIHost + "/{0}/{1}", ServiceType.ToString(), MethodType.managerLogin);
            req.postResponse(pars, url);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    CustomerList cl = Json.Deserialize<CustomerList>(req.data);
                    if (null == cl)
                        break;
                    managerTokenList = cl.customerList;
                    res.Failures = cl.failures;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
        #endregion

        #region Json解析辅助类
        /// <summary>
        /// Campaign Id 列表
        /// </summary>
        [Serializable]
        private class MCampaignIdList : ApiResult
        {
            /// <summary>
            /// Gets or sets the campaign id list.
            /// </summary>
            /// <value>The campaign id list.</value>
            public long[] campaignIdList { get; set; }
        }

        /// <summary>
        /// 生成文件fileId
        /// </summary>
        [Serializable]
        private class AccountFile : ApiResult
        {
            public string fileId { get; set; }
        }

        /// <summary>
        ///整账户下载FileState信息
        /// </summary>
        [Serializable]
        private class FileStateInfo : ApiResult
        {
            public string isGenerated { get; set; }
            public string filePath { get; set; }
            public string fileMd5 { get; set; }
            public long fileSize { get; set; }
        }
        /// <summary>
        /// 排除ip
        /// </summary>
        [Serializable]
        private class ExcludeIpList : ApiResult
        {
            public string[] excludeIpList { get; set; }
        }
        /// <summary>
        /// 客服用户列表
        /// </summary>
        [Serializable]
        private class CustomerList : ApiResult
        {
            public List<LoginToken> customerList { get; set; }
        }
        #endregion
    }
}
