﻿using System;
using System.Collections.Generic;
using Pz.API.QH360.SDK.Common;
using Pz.API.QH360.SDK.Enum;
using Pz.API.QH360.SDK.Model;

namespace Pz.API.QH360.SDK.API
{
    /// <summary>
    /// 计划服务
    /// </summary>
    public class APICampaign : APIParent
    {
        /// <summary>
        /// 设置服务的类型
        /// </summary>
        internal override ServiceType ServiceType
        {
            get { return ServiceType.campaign; }
        }

        #region 服务接口类
        /// <summary>
        /// 添加推广计划
        /// </summary>
        /// <param name="campaign">计划实体
        /// name  	推广计划名称 
        ///budget   推广计划每日预算，默认为0，表示不限制 
        ///region   投放地域，json格式的地域id数组，默认为[0]，代表全地域投放
        ///schedule    json 格式，Week字段1-7 表示周一至周日，若其中一星期某天不投放，可跳过，hour 总数为24   其中0为此时段不投放，1为投放。例: 
        ///[{“week”:1,”hour”:[1,1,1,1,1,1,1,1,1,1……]},{“week”:3,”hour”:[1,0,1,1,1,0,1,1,1,1……]}……]每周二不投放。 
        ///startDate  推广计划开始日期，默认为添加之日开始 
        ///endDate    推广计划结束日期，默认永久有效 </param>
        /// <param name="campaignid">计划实体id</param>
        /// <returns>Result.</returns>
        /// <exception cref="System.Exception">
        /// campaign is null
        /// </exception>
        public Result add(QH360MCampaign campaign, ref long campaignid)
        {
            if (null == campaign)
                throw new Exception("campaign is null");
            List<MyPostParameter> parameters = new List<MyPostParameter>();
            parameters.Add(new MyPostParameter("name", campaign.name));
            if (campaign.budget.HasValue)
                parameters.Add(new MyPostParameter("budget", (int)campaign.budget));
            if (!string.IsNullOrEmpty(campaign.region))
                parameters.Add(new MyPostParameter("region", campaign.region));
            if (!string.IsNullOrEmpty(campaign.schedule))
                parameters.Add(new MyPostParameter("schedule", campaign.schedule));
            if (!string.IsNullOrEmpty(campaign.startDate))
                parameters.Add(new MyPostParameter("startDate", string.Format(new CustomDateFormatter(), "{0:S}", campaign.startDate)));
            if (!string.IsNullOrEmpty(campaign.endDate))
                parameters.Add(new MyPostParameter("endDate", string.Format(new CustomDateFormatter(), "{0:S}", campaign.endDate)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.MaxExceptionCount = 1;
            req.methodType = MethodType.add;
            req.postResponse(parameters);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MCampaignIdItem m = Json.Deserialize<MCampaignIdItem>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    campaignid = m.id;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 根据计划id 删除相应的推广计划，删除推广计划时，自动将下属的推广组、创意、关键词删除。
        /// </summary>
        /// <param name="id">计划id</param>
        /// <param name="affectedRecords">影响行数.</param>
        /// <returns>Result.</returns>
        public Result deleteById(long id, ref int affectedRecords)
        {
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.deleteById;
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("id", id));
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    AffectedRecords apiRes = Json.Deserialize<AffectedRecords>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    affectedRecords = apiRes.affectedRecords;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 根据计划id 更新相应的推广计划
        /// </summary>
        /// <param name="campaign">计划实体
        /// name  	推广计划名称 
        ///budget   推广计划每日预算，默认为0，表示不限制 
        ///region   投放地域，json格式的地域id数组，默认为[0]，代表全地域投放
        ///schedule    json 格式，Week字段1-7 表示周一至周日，若其中一星期某天不投放，可跳过，hour 总数为24   其中0为此时段不投放，1为投放。例: 
        ///[{“week”:1,”hour”:[1,1,1,1,1,1,1,1,1,1……]},{“week”:3,”hour”:[1,0,1,1,1,0,1,1,1,1……]}……]每周二不投放。 
        ///startDate  推广计划开始日期，默认为添加之日开始 
        ///endDate    推广计划结束日期，默认永久有效 
        /// status    状态 </param>
        /// <param name="campaignid">计划id</param>
        /// <returns>Result.</returns>
        /// <exception cref="System.Exception">campaign is null</exception>
        public Result update(QH360MCampaign campaign, ref long campaignid)
        {
            if (null == campaign)
                throw new Exception("campaign is null");
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("id", campaign.id));
            if (!string.IsNullOrEmpty(campaign.name))
                pars.Add(new MyPostParameter("name", campaign.name));
            if (campaign.budget.HasValue)
                pars.Add(new MyPostParameter("budget", (int)campaign.budget));
            if (null != campaign.region)
                pars.Add(new MyPostParameter("region", campaign.region));
            if (null != campaign.schedule)
                pars.Add(new MyPostParameter("schedule", campaign.schedule));
            if (null != campaign.startDate)
                pars.Add(new MyPostParameter("startDate", string.Format(new CustomDateFormatter(), "{0:S}", campaign.startDate)));
            if (null != campaign.endDate)
                pars.Add(new MyPostParameter("endDate", string.Format(new CustomDateFormatter(), "{0:S}", campaign.endDate)));
            if (null != campaign.status)
                pars.Add(new MyPostParameter("status", campaign.status));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.update;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MCampaignIdItem apiRes = Json.Deserialize<MCampaignIdItem>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    campaignid = apiRes.id;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 根据一个推广计划id，获取该推广计划的信息
        /// </summary>
        /// <param name="id">计划id</param>
        /// <param name="campaign">计划实体</param>
        /// <returns>Result.</returns>
        public Result getInfoById(long id, ref QH360MCampaign campaign)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("id", id));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.methodType = MethodType.getInfoById;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    QH360MCampaign apiRes = Json.Deserialize<QH360MCampaign>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    campaign = apiRes;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
        /// <summary>
        /// 批量获取推广计划的信息
        /// </summary>
        /// <param name="idList">json格式的id数组，如[1000001,20000001,3000001]</param>
        /// <param name="liCampaigns">计划实体列表</param>
        /// <returns>Result.</returns>
        public Result getInfoByIdList(long[] idList, ref List<QH360MCampaign> liCampaigns)
        {
            if (idList == null)
                idList = new long[0];
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.getInfoByIdList;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MCampaignList apiRes = Json.Deserialize<MCampaignList>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    liCampaigns = apiRes.campaignList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
        #endregion

        #region Json解析辅助类
        /// <summary>
        /// Campaign Item 对象
        /// </summary>
        [Serializable]
        private class MCampaignIdItem : ApiResult
        {
            public long id { get; set; }
        }

        /// <summary>
        /// 计划列表
        /// </summary>
        [Serializable]
        private class MCampaignList : ApiResult
        {
            public List<QH360MCampaign> campaignList { get; set; }
        }
        #endregion
    }


}
