﻿using System;
using System.Collections.Generic;

using Pz.API.QH360.SDK.Common;
using Pz.API.QH360.SDK.Enum;
using Pz.API.QH360.SDK.Model;

namespace Pz.API.QH360.SDK.API
{
    /// <summary>
    /// 创意服务
    /// </summary>
    public class APICreative : APIParent
    {
        /// <summary>
        /// 设置服务的类型
        /// </summary>
        internal override ServiceType ServiceType
        {
            get { return ServiceType.creative; }
        }

        #region 服务接口类
        /// <summary>
        /// 根据广告组id 查询所有或满足条件的创意id
        /// </summary>
        /// <param name="groupId">广告组id </param>
        /// <param name="status">创意状态，enable 表示启用，pause 表示暂停，pending表示待审核，reject表示审核不通过 </param>
        /// <param name="creativeIds">创意id列表</param>
        /// <returns>Result.</returns>
        public Result getIdListByGroupId(long groupId, string status, ref long[] creativeIds)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("groupId", groupId));
            if (!string.IsNullOrEmpty(status))
                pars.Add(new MyPostParameter("status", status));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            
            req.methodType = MethodType.getIdListByGroupId;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MCreativeIdList m = Json.Deserialize<MCreativeIdList>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    creativeIds = m.creativeIdList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 添加一个创意
        /// </summary>
        /// <param name="creative">创意实体
        /// groupId  广告组id 
        ///title   广告标题 
        ///description1   广告描述1 
        ///description2  广告描述2 
        ///destinationUrl   链接网址 
        ///displayUrl  显示网址，如果不填，默认为链接网址的域名</param>
        /// <param name="creativeid">创意id</param>
        /// <returns>Result.</returns>
        /// <exception cref="System.Exception">
        /// creative is null
        /// </exception>
        public Result add(QH360MCreative creative, ref long creativeid)
        {
            if (null == creative)
                throw new Exception("creative is null");
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("groupId", creative.groupId));
            pars.Add(new MyPostParameter("title", creative.title));
            pars.Add(new MyPostParameter("description1", creative.description1));
            pars.Add(new MyPostParameter("destinationUrl", creative.destinationUrl));
            if (!string.IsNullOrEmpty(creative.description2))
                pars.Add(new MyPostParameter("description2", creative.description2));
            if (!string.IsNullOrEmpty(creative.displayUrl))
                pars.Add(new MyPostParameter("displayUrl", creative.displayUrl));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.MaxExceptionCount = 1;
            req.methodType = MethodType.add;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MCreativeIdItem aur = Json.Deserialize<MCreativeIdItem>(req.data);
                    if (null == aur)
                        break;
                    res.Failures = aur.failures;
                    creativeid = aur.id;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 删除一条创意
        /// </summary>
        /// <param name="id">创意id</param>
        /// <param name="affectedRecords">影响行数</param>
        /// <returns></returns>
        public Result deleteById(long id, ref int affectedRecords)
        {
            return deleteByIdList(new long[] { id }, ref affectedRecords);
        }
        /// <summary>
        /// 删除创意
        /// </summary>
        /// <param name="idList">json格式的id数组，如[1000001,20000001,3000001] </param>
        /// <param name="affectedRecords">影响的行数</param>
        /// <returns>Result.</returns>
        public Result deleteByIdList(long[] idList, ref int affectedRecords)
        {
            if (idList == null)
                idList = new long[0];
            Result res = new Result();
            affectedRecords = 0;
            int maxRequestCount = MyConfig.MaxRequestCount;
            int count = idList.Length / maxRequestCount;
            if (idList.Length % maxRequestCount != 0)
                count += 1;
            for (int i = 0; i < count; i++)
            {
                int startIndex = i * maxRequestCount;
                int endIndex = maxRequestCount * (i + 1) - 1;
                int length = maxRequestCount;
                if (endIndex > idList.Length - 1)
                    length = idList.Length - startIndex;

                long[] arrRange = new long[length];
                Array.Copy(idList, startIndex, arrRange, 0, length);
                int affectedNum = 0;

                List<MyPostParameter> pars = new List<MyPostParameter>();
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(arrRange)));
                MyHttpRequest req = new MyHttpRequest(this);

                req.methodType = MethodType.deleteByIdList;
                req.postResponse(pars);
                req.GetRes(ref res);
                switch (req.status)
                {
                    case ResultStatus.Succeed:
                        AffectedRecords apiRes = Json.Deserialize<AffectedRecords>(req.data);
                        if (null == apiRes)
                            break;
                        res.Failures = apiRes.failures;
                        affectedNum = apiRes.affectedRecords;
                        break;
                    case ResultStatus.Fail:
                        break;
                    case ResultStatus.DataIsEmpty:
                        break;
                }
                affectedRecords += affectedNum;
            }
            return res;
        }

        /// <summary>
        /// 修改创意
        /// </summary>
        /// <param name="id">创意id  </param>
        /// <param name="status">enable表示启用，pause表示暂停，一个请求中，无法同时修改status和创意其他字段，否则会忽略状态的修改。</param>
        /// <param name="title">广告标题 </param>
        /// <param name="description1">广告描述1，修改创意的描述，必须同时传新的描述1和新的描述2，如果只传描述1，则等效于更新描述1并删除原来的描述2</param>
        /// <param name="description2">广告描述2，如果要修改描述2，必须同时传描述1，否则忽略该修改</param>
        /// <param name="destinationUrl">链接网址</param>
        /// <param name="displayUrl">显示网址，如果不填，默认为链接网址的域名</param>
        /// <param name="creativeid">创意id</param>
        /// <returns></returns>
        public Result update(long id, string status, string title, string description1, string description2, string destinationUrl, string displayUrl, ref long creativeid)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("id", id));
            if (null != status)
                pars.Add(new MyPostParameter("status", status));
            if (null != title)
                pars.Add(new MyPostParameter("title", title));
            if (null != description1)
                pars.Add(new MyPostParameter("description1", description1));
            if (null != description2)
                pars.Add(new MyPostParameter("description2", description2));
            if (null != destinationUrl)
                pars.Add(new MyPostParameter("destinationUrl", destinationUrl));
            if (null != displayUrl)
                pars.Add(new MyPostParameter("displayUrl", displayUrl));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.update;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MCreativeIdItem aur = Json.Deserialize<MCreativeIdItem>(req.data);
                    if (null == aur)
                        break;
                    res.Failures = aur.failures;
                    creativeid = aur.id;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 批量更新创意的某个字段值
        /// </summary>
        /// <param name="idList">json格式的id数组，如[1000001,20000001,3000001] </param>
        /// <param name="status">enable表示启用，pause表示暂停，对指定的所有id生效</param>
        /// <param name="title">广告标题 </param>
        /// <param name="destinationUrl">链接地址</param>
        /// <param name="displayUrl">显示地址</param>
        /// <param name="affectedRecords">影响的行数</param>
        /// <returns>Result.</returns>
        /// <exception cref="System.Exception">idList is null</exception>
        public Result updateByIdList(long[] idList, string status, string title, string destinationUrl, string displayUrl, ref int affectedRecords)
        {
            if (idList == null)
                idList = new long[0];
            Result res = new Result();
            affectedRecords = 0;
            int maxRequestCount = MyConfig.MaxRequestCount;
            int count = idList.Length / maxRequestCount;
            if (idList.Length % maxRequestCount != 0)
                count += 1;
            for (int i = 0; i < count; i++)
            {
                int startIndex = i * maxRequestCount;
                int endIndex = maxRequestCount * (i + 1) - 1;
                int length = maxRequestCount;
                if (endIndex > idList.Length - 1)
                    length = idList.Length - startIndex;
                long[] arrRange = new long[length];
                Array.Copy(idList, startIndex, arrRange, 0, length);
                int affectedNum = 0;
                List<MyPostParameter> pars = new List<MyPostParameter>();
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(arrRange)));
                if (null!=status)
                    pars.Add(new MyPostParameter("status", status));
                if (null != title)
                    pars.Add(new MyPostParameter("title", title));
                if (null != destinationUrl)
                    pars.Add(new MyPostParameter("destinationUrl", destinationUrl));
                if (null != displayUrl)
                    pars.Add(new MyPostParameter("displayUrl", displayUrl));
                MyHttpRequest req = new MyHttpRequest(this);

                req.methodType = MethodType.updateByIdList;
                req.postResponse(pars);
                req.GetRes(ref res);
                switch (req.status)
                {
                    case ResultStatus.Succeed:
                        AffectedRecords apiRes = Json.Deserialize<AffectedRecords>(req.data);
                        if (null == apiRes)
                            break;
                        res.Failures = apiRes.failures;
                        affectedRecords = apiRes.affectedRecords;
                        break;
                    case ResultStatus.Fail:
                        break;
                    case ResultStatus.DataIsEmpty:
                        break;
                }
                affectedRecords += affectedNum;
            }
            return res;
        }
        /// <summary>
        /// 获取创意详细信息
        /// </summary>
        /// <param name="id">创意id</param>
        /// <param name="creative">创意实体</param>
        /// <returns>Result.</returns>
        public Result getInfoById(long id, ref QH360MCreative creative)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("id", id));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            
            req.methodType = MethodType.getInfoById;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    QH360MCreative apiRes = Json.Deserialize<QH360MCreative>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    creative = apiRes;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 批量获取创意详细信息
        /// </summary>
        /// <param name="idList">json格式的创意id数组，如[2,3]</param>
        /// <param name="liCreatives">创意实体列表</param>
        /// <returns>Result.</returns>
        public Result getInfoByIdList(long[] idList, ref List<QH360MCreative> liCreatives)
        {
            if (idList == null)
                idList = new long[0];
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            
            req.methodType = MethodType.getInfoByIdList;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MCreativeList apiRes = Json.Deserialize<MCreativeList>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    liCreatives = apiRes.creativeList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
        /// <summary>
        /// 批量获取创意状态 
        /// </summary>
        /// <param name="idList">要获取状态的创意id列表</param>
        /// <param name="liCreatives">返回状态model</param>
        /// <returns></returns>
        public Result getStatusByIdList(long[] idList, ref List<QH360MCreative> liCreatives)
        {
            if (idList == null)
                idList = new long[0];
            Result res = new Result();
            liCreatives = new List<QH360MCreative>();
            int maxRequestCount = MyConfig.MaxRequestCount;
            int count = idList.Length / maxRequestCount;
            if (idList.Length % maxRequestCount != 0)
                count += 1;
            for (int i = 0; i < count; i++)
            {
                int startIndex = i * maxRequestCount;
                int endIndex = maxRequestCount * (i + 1) - 1;
                int length = maxRequestCount;
                if (endIndex > idList.Length - 1)
                    length = idList.Length - startIndex;
                long[] arrRange = new long[length];
                Array.Copy(idList, startIndex, arrRange, 0, length);
                List<MyPostParameter> pars = new List<MyPostParameter>();
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
                MyHttpRequest req = new MyHttpRequest(this);

                req.methodType = MethodType.getStatusByIdList;
                req.postResponse(pars);
                req.GetRes(ref res);
                switch (req.status)
                {
                    case ResultStatus.Succeed:
                        MCreativeList apiRes = Json.Deserialize<MCreativeList>(req.data);
                        if (null == apiRes)
                            break;
                        res.Failures = apiRes.failures;
                        if (apiRes.creativeList != null)
                            liCreatives.AddRange(apiRes.creativeList);
                        break;
                    case ResultStatus.Fail:
                        break;
                    case ResultStatus.DataIsEmpty:
                        break;
                }
            }
            return res;
        }
        /// <summary>
        /// 询某个开始时间至当前时间所有被编辑过（新增、修改、删除）的创意 id列表
        /// </summary>
        /// <param name="fromTime">起始时间</param>
        /// <param name="idList">返回的id列表</param>
        /// <returns></returns>
        public Result getChangedIdList(string fromTime, ref long[] idList)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            if (!string.IsNullOrEmpty(fromTime))
                pars.Add(new MyPostParameter("fromTime", string.Format(new CustomDateFormatter(), "{0:L}", fromTime)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.methodType = MethodType.getChangedIdList;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MCreativeIdList m = Json.Deserialize<MCreativeIdList>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    idList = m.creativeIdList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        #endregion


        #region Json解析辅助类
        /// <summary>
        /// Creative Id 列表
        /// </summary>
        [Serializable]
        internal class MCreativeIdList : ApiResult
        {
            public long[] creativeIdList { get; set; }
        }
        /// <summary>
        /// Creative Item 对象
        /// </summary>
        [Serializable]
        internal class MCreativeIdItem : ApiResult
        {
            public long id { get; set; }
        }

        /// <summary>
        /// Creative Id 列表
        /// </summary>
         [Serializable]
        internal class MCreativeList : ApiResult
        {
            public List<QH360MCreative> creativeList { get; set; }
        }
        #endregion

    }
}
