﻿using System;
using System.Collections.Generic;
using Pz.API.QH360.SDK.Common;
using Pz.API.QH360.SDK.Enum;
using Pz.API.QH360.SDK.Model;
namespace Pz.API.QH360.SDK.API
{
    /// <summary>
    /// 组服务
    /// </summary>
   public class APIGroup:APIParent
    {
        /// <summary>
        /// 设置服务的类型
        /// </summary>
        internal override ServiceType ServiceType
        {
            get
            {
                return ServiceType.group;
            }
        }

        #region 服务接口类
        /// <summary>
        /// 根据推广计划id 查询所有或满足条件的广告组id
        /// </summary>
        /// <param name="campaignId">推广计划id </param>
        /// <param name="status">广告组状态，enable表示启用，pause表示暂停</param>
        /// <param name="fromTime">广告组最后更新时间，值不为空时，代表查询最后更新时间大于该值的结果 </param>
        /// <param name="groupIds">The group ids.</param>
        /// <returns>Result.</returns>
        public Result getIdListByCampaignId(long campaignId, string status, string fromTime, ref long[] groupIds)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("campaignId", campaignId));
            if (!string.IsNullOrEmpty(status))
                pars.Add(new MyPostParameter("status", status));
             if (!string.IsNullOrEmpty(fromTime))
                pars.Add(new MyPostParameter("fromTime", string.Format(new CustomDateFormatter(), "{0:L}",fromTime)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.methodType = MethodType.getIdListByCampaignId;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MGroupIdList m = Json.Deserialize<MGroupIdList>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    groupIds = m.groupIdList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 添加一个广告组
        /// </summary>
        /// <param name="group">组实体
        ///campaignId  推广计划id 
        ///name  广告组名称 
        ///price  广告组出价 
        ///negativeWords  匹配方式为短语匹配或者精确匹配的否定关键词,使用JSON格式：其中phrase 代表短语 exact 代表精确  {”phrase”:[”短语匹配1”],”exact”:[”精确1”,”精确2”]}</param>
        /// <param name="groupid">组id</param>
        /// <returns>Result.</returns>
        /// <exception cref="System.Exception">
        /// group is null
        /// </exception>
        public Result add(QH360MGroup group, ref long groupid)
        {
            if (null == group)
                throw new Exception("group is null");
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("campaignId", group.campaignId));
            pars.Add(new MyPostParameter("name", group.name));
            pars.Add(new MyPostParameter("price", group.price));
            NegativeWordsType nwt = new NegativeWordsType();
            nwt.phrase = string.IsNullOrEmpty(group.negativeWords) ? new string[0] : group.negativeWords.Split(',');
            nwt.exact = string.IsNullOrEmpty(group.exactNegativeWords) ? new string[0] : group.exactNegativeWords.Split(',');
            pars.Add(new MyPostParameter("negativeWords", Json.SerializeObject(nwt)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.MaxExceptionCount = 1;
            req.methodType = MethodType.add;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MGroupIdItem aur = Json.Deserialize<MGroupIdItem>(req.data);
                    if (null == aur)
                        break;
                    res.Failures = aur.failures;
                    groupid = aur.id;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 根据id 删除相应的推广组，删除推广组时，自动将下属的创意、关键词删除。
        /// </summary>
        /// <param name="id">广告组id</param>
        /// <param name="affectedRecords">影响的行数</param>
        /// <returns>Result.</returns>
        public Result deleteById(long id, ref int affectedRecords)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("id", id));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            
            req.methodType = MethodType.deleteById;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    AffectedRecords apiRes = Json.Deserialize<AffectedRecords>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    affectedRecords = apiRes.affectedRecords;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 修改一个广告组
        /// </summary>
        /// <param name="group">广告组实体
        /// id   广告组id
        ///name  广告组名称 
        ///price  广告组出价 
        ///negativeWords  匹配方式为短语匹配或者精确匹配的否定关键词,使用JSON格式：其中phrase 代表短语 exact 代表精确  {”phrase”:[”短语匹配1”],”exact”:[”精确1”,”精确2”]}</param>
        /// <param name="groupid">广告组id</param>
        /// <returns>Result.</returns>
        /// <exception cref="System.Exception">group is null</exception>
        public Result update(QH360MGroup group, ref long groupid)
        {
            if (null == group)
                throw new Exception("group is null");
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("id", group.id));
            if (!string.IsNullOrEmpty(group.status))
                pars.Add(new MyPostParameter("status", group.status));
            if (null != group.name)
                pars.Add(new MyPostParameter("name", group.name));
            if (group.price > 0.0)
                pars.Add(new MyPostParameter("price", group.price));
            NegativeWordsType nwt = new NegativeWordsType();
            nwt.phrase = string.IsNullOrEmpty(group.negativeWords) ? new string[0] : group.negativeWords.Split(',');
            nwt.exact = string.IsNullOrEmpty(group.exactNegativeWords) ? new string[0] : group.exactNegativeWords.Split(',');
            pars.Add(new MyPostParameter("negativeWords", Json.SerializeObject(nwt)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.methodType = MethodType.update;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MGroupIdItem aur = Json.Deserialize<MGroupIdItem>(req.data);
                    if (null == aur)
                        break;
                    res.Failures = aur.failures;
                    groupid = aur.id;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 根据一个广告组id，获取该广告组的详细信息
        /// </summary>
        /// <param name="id">广告组id</param>
        /// <param name="group">广告组实体
        ///id  广告组的id 
        ///campaignId  广告计划的id 
        ///name  广告组的名称 
        ///price  广告组默认出价 
        ///negativeWords  短语匹配的否定关键词 
        ///exactNegativeWords 精确匹配的否定关键词
        ///status  广告组状态值，enable表示启用，pause表示暂停 
        ///updateTime  广告组最后更新时间 </param>
        /// <returns>Result.</returns>
        public Result getInfoById(long id, ref QH360MGroup group)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("id",id));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            
            req.methodType = MethodType.getInfoById;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    QH360MGroup apiRes = Json.Deserialize<QH360MGroup>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    group = apiRes;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
       /// <summary>
        /// 批量获取该广告组的信息
       /// </summary>
        /// <param name="idList">json格式的id数组，如[1000001,20000001,3000001] </param>
       /// <param name="liGroups">广告组实体列表</param>
       /// <returns></returns>
        public Result getInfoByIdList(long[] idList, ref List<QH360MGroup> liGroups)
        {
            if (idList == null)
                idList = new long[0];
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.getInfoByIdList;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MGroupList apiRes = Json.Deserialize<MGroupList>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    liGroups = apiRes.groupList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

       /// <summary>
        /// 添加多个广告组
       /// </summary>
        /// <param name="campaignId">推广计划id </param>
       /// <param name="groups">广告组项
       /// campaignId 推广计划id 
        ///groups   广告组信息（名字和价格），示例： [{"name":"ceshi1","price":1},{"name":" ceshi2","price":1}] </param>
       /// <param name="groupIds">广告组id列表</param>
       /// <returns></returns>
        private Result bulkAdd(long campaignId, QH360MGroup.GroupItem[] groups, ref long[] groupIds)
        {
            if (groups == null)
                groups = new QH360MGroup.GroupItem[0];
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("campaignId", campaignId));
            pars.Add(new MyPostParameter("groups", Json.SerializeObject(groups)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.MaxExceptionCount = 1;
            req.methodType = MethodType.bulkAdd;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MGroupIdList apiRes = Json.Deserialize<MGroupIdList>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    groupIds = apiRes.groupIdList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
       /// <summary>
        /// 添加多个广告组
       /// </summary>
        /// <param name="campaignId">推广计划id </param>
       /// <param name="liGroups">广告组实体列表</param>
       /// <param name="groupIds">广告组id列表</param>
       /// <returns></returns>
        public Result bulkAdd(long campaignId, List<QH360MGroup> liGroups, ref long[] groupIds)
        {
            if (liGroups == null)
                liGroups = new List<QH360MGroup>();
            QH360MGroup.GroupItem[] groups = new QH360MGroup.GroupItem[liGroups.Count];
            for (int i = 0; i < liGroups.Count; i++)
            {
                QH360MGroup group = liGroups[i];
                QH360MGroup.GroupItem groupItem = new QH360MGroup.GroupItem();
                groupItem.name = group.name;
                groupItem.price = group.price;
                groups[i] = groupItem;
            }
           return  bulkAdd(campaignId, groups, ref groupIds);
        }

        #endregion

        #region Json解析辅助类
        /// <summary>
        /// Group Id 列表
        /// </summary>
        [Serializable]
       internal class MGroupIdList : ApiResult
        {
            public long[] groupIdList { get; set; }
        }
        /// <summary>
        /// Group Item 对象
        /// </summary>
        [Serializable]
       internal class MGroupIdItem:ApiResult
        {
            public long id { get; set; }
        }

       /// <summary>
       /// 组列表
       /// </summary>
        [Serializable]
        internal class MGroupList : ApiResult
        {
            public List<QH360MGroup> groupList { get; set; }
        }
        #endregion
    }
}
