﻿using System;
using System.Collections.Generic;

using Pz.API.QH360.SDK.Common;
using Pz.API.QH360.SDK.Enum;
using Pz.API.QH360.SDK.Model;

namespace Pz.API.QH360.SDK.API
{
    /// <summary>
    /// 关键词服务
    /// </summary>
    public class APIKeyword : APIParent
    {
        /// <summary>
        /// 设置服务的类型
        /// </summary>
        /// <value>The type of the service.</value>
        internal override ServiceType ServiceType
        {
            get
            {
                return ServiceType.keyword;
            }
        }

        #region 服务接口类
        /// <summary>
        /// 根据广告组id 查询所有或满足条件的关键词id
        /// </summary>
        /// <param name="groupId">广告组id </param>
        /// <param name="status">关键词状态，enable表示启用，pause表示暂停，pending表示待审核，reject表示审核不通过</param>
        /// <param name="matchType">关键词匹配方式：exact表示精确，phrase表示短语，broad表示广泛匹配 .</param>
        /// <param name="keywordIds">关键词id列表</param>
        /// <returns>Result.</returns>
        public Result getIdListByGroupId(long groupId, string status, string matchType, ref long[] keywordIds)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("groupId", groupId));
            if (!string.IsNullOrEmpty(status))
                pars.Add(new MyPostParameter("status", status));
            if (!string.IsNullOrEmpty(matchType))
                pars.Add(new MyPostParameter("matchType", matchType));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.getIdListByGroupId;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MKeywordIdList m = Json.Deserialize<MKeywordIdList>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    keywordIds = m.keywordIdList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 添加关键词
        /// </summary>
        /// <param name="groupId">广告组id</param>
        /// <param name="keywords">关键词实体</param>
        /// <param name="keywordIds">关键词id列表</param>
        /// <returns>Result.</returns>
        /// <exception cref="System.Exception">keywords is null</exception>
        private Result add(long groupId, QH360MKeyword.Keywords keywords, ref long[] keywordIds)
        {
            if (null == keywords)
                throw new Exception("keywords is null");
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("groupId", groupId));
            pars.Add(new MyPostParameter("keywords", Json.SerializeObject(keywords)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.MaxExceptionCount = 1;
            req.methodType = MethodType.add;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MKeywordIdList aur = Json.Deserialize<MKeywordIdList>(req.data);
                    if (null == aur)
                        break;
                    res.Failures = aur.failures;
                    keywordIds = aur.keywordIdList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 添加单个关键词
        /// </summary>
        /// <param name="keyword">关键词实体</param>
        /// <param name="keywordId">关键词id</param>
        /// <returns></returns>
        public Result add(QH360MKeyword keyword, ref long keywordId)
        {
            QH360MKeyword.Keywords keywords = new QH360MKeyword.Keywords();
            if (null == keyword)
                throw new Exception("keyword is null");
            if (string.IsNullOrEmpty(keyword.matchType))
                throw new Exception("keyword matchType is null");
            QH360MKeyword.KeywordItem keywordItem = null;
            List<QH360MKeyword.KeywordItem> liphrase = new List<QH360MKeyword.KeywordItem>();
            List<QH360MKeyword.KeywordItem> liexact = new List<QH360MKeyword.KeywordItem>();
            List<QH360MKeyword.KeywordItem> libroad = new List<QH360MKeyword.KeywordItem>();
            keywordItem = new QH360MKeyword.KeywordItem();
            keywordItem.word = keyword.word;
            keywordItem.price = keyword.price;
            keywordItem.url = keyword.destinationUrl;
            string type = keyword.matchType.ToLower();
            switch (type)
            {
                case "exact":
                    liexact.Add(keywordItem);
                    break;
                case "phrase":
                    liphrase.Add(keywordItem);
                    break;
                case "broad":
                    libroad.Add(keywordItem);
                    break;
            }
            keywords.exact = liexact.ToArray();
            keywords.phrase = liphrase.ToArray();
            keywords.broad = libroad.ToArray();
            long[] keywordIds = new long[0];
            Result res = add(keyword.groupId, keywords, ref keywordIds);
            if (keywordIds != null && keywordIds.Length > 0)
                keywordId = keywordIds[0];
            return res;
        }

        /// <summary>
        /// 批量添加关键词
        /// </summary>
        /// <param name="groupId">广告组id</param>
        /// <param name="liKeyword">关键词实体列表</param>
        /// <param name="keywordIds">关键词id</param>
        /// <returns>Result.</returns>
        public Result add(long groupId, List<QH360MKeyword> liKeyword, ref long[] keywordIds)
        {
            return add(groupId, liKeyword, ref keywordIds, null);
        }
         /// <summary>
        /// 批量添加关键词
        /// </summary>
        /// <param name="groupId">广告组id</param>
        /// <param name="liKeyword">关键词实体列表</param>
        /// <param name="keywordIds">关键词id</param>
        /// <param name="reportsProgress">批量操作实时返回成功记录数量的委托</param>
        /// <returns></returns>
        public Result add(long groupId, List<QH360MKeyword> liKeyword, ref long[] keywordIds, WorkReportsProgressDelegate reportsProgress)
        {
            if (liKeyword == null)
                liKeyword = new List<QH360MKeyword>();
            //首先对传过来的按照匹配方式进行排序
            liKeyword.Sort();
            Result res = new Result();
            List<Failure> liFailure = new List<Failure>();
            List<long> liKeywordId = new List<long>();
            int maxRequestCount = MyConfig.MaxBatchCount;
            int count = liKeyword.Count / maxRequestCount;
            if (liKeyword.Count % maxRequestCount != 0)
                count += 1;
            QH360MKeyword.Keywords keywords = null;
            long[] keywordIdsTemp = new long[0];
            for (int i = 0; i < count; i++)
            {
                keywords = new QH360MKeyword.Keywords();
                List<QH360MKeyword.KeywordItem> liphrase = new List<QH360MKeyword.KeywordItem>();
                List<QH360MKeyword.KeywordItem> liexact = new List<QH360MKeyword.KeywordItem>();
                List<QH360MKeyword.KeywordItem> libroad = new List<QH360MKeyword.KeywordItem>();
                QH360MKeyword.KeywordItem keywordItem = null;
                int startIndex = i * maxRequestCount;
                int endIndex = maxRequestCount * (i + 1) - 1;
                for (int j = startIndex; j <= endIndex && j < liKeyword.Count; j++)
                {
                    QH360MKeyword keyword = liKeyword[j];
                    keywordItem = new QH360MKeyword.KeywordItem();
                    keywordItem.word = keyword.word;
                    keywordItem.price = keyword.price;
                    keywordItem.url = keyword.destinationUrl;
                    string type = keyword.matchType;
                    if (!string.IsNullOrEmpty(type))
                    {
                        type = type.ToLower();
                        switch (type)
                        {
                            case "exact":
                                liexact.Add(keywordItem);
                                break;
                            case "phrase":
                                liphrase.Add(keywordItem);
                                break;
                            case "broad":
                                libroad.Add(keywordItem);
                                break;
                        }
                    }
                }
                keywords.exact = liexact.ToArray();
                keywords.phrase = liphrase.ToArray();
                keywords.broad = libroad.ToArray();
                res = add(groupId, keywords, ref keywordIdsTemp);
                if (res.Failures != null)
                    liFailure.AddRange(res.Failures);
                if (keywordIdsTemp != null)
                {
                    liKeywordId.AddRange(keywordIdsTemp);
                    if (reportsProgress != null)
                    {
                        //只计算成功的个数
                        int successCount = 0;
                        for (int j = 0; j < keywordIdsTemp.Length; j++)
                        {
                            if (!keywordIdsTemp[j].Equals(0))
                                successCount++;
                        }
                        reportsProgress(successCount);
                    }
                }
            }
            keywordIds = liKeywordId.ToArray();
            Failure[] failure = null;
            if (liFailure.Count > 0)
                failure = liFailure.ToArray();
            res.Failures = failure;
            return res;
        }

        /// <summary>
        /// 关键词更新
        /// </summary>
        /// <param name="id">关键词id </param>
        /// <param name="status">enable表示启用，pause表示暂停，不能同时修改status和其他字段，否则修改无效。</param>
        /// <param name="price">关键词价格</param>
        /// <param name="url">链接网址</param>
        /// <param name="matchType">匹配方式，exact表示精确，phrase表示短语，broad表示广泛匹配。 </param>
        /// <param name="keywordId">关键词id</param>
        /// <returns>Result.</returns>
        public Result update(long id, string status, double? price, string url, string matchType, ref long keywordId)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("id", id));
            if (null != status)
                pars.Add(new MyPostParameter("status", status));
            if (price.HasValue)
                pars.Add(new MyPostParameter("price", price));
            if (null != url)
                pars.Add(new MyPostParameter("url", url));
            if (null != matchType)
                pars.Add(new MyPostParameter("matchType", matchType));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.update;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MKeywordIdItem aur = Json.Deserialize<MKeywordIdItem>(req.data);
                    if (null == aur)
                        break;
                    res.Failures = aur.failures;
                    keywordId = aur.id;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }

        /// <summary>
        /// 批量更新关键词的某个字段值
        /// </summary>
        /// <param name="idList">json格式的关键词id数组，如[1000001,20000001]</param>
        /// <param name="status">enable表示启用，pause表示暂停，对idList里的所有id生效</param>
        /// <param name="price">关键词价格.</param>
        /// <param name="url">链接网址 </param>
        /// <param name="matchType">匹配方式，exact表示精确，phrase表示短语，broad表示广泛匹配。</param>
        /// <param name="affectedRecords">影响行数.</param>
        /// <returns>Result.</returns>
        public Result updateByIdList(long[] idList, string status, double? price, string url, string matchType, ref int affectedRecords)
        {
            if (idList == null)
                idList = new long[0];
            Result res = new Result();
            affectedRecords = 0;
            int maxRequestCount = MyConfig.MaxRequestCount;
            int count = idList.Length / maxRequestCount;
            if (idList.Length % maxRequestCount != 0)
                count += 1;
            for (int i = 0; i < count; i++)
            {
                int startIndex = i * maxRequestCount;
                int endIndex = maxRequestCount * (i + 1) - 1;
                int length = maxRequestCount;
                if (endIndex > idList.Length - 1)
                    length = idList.Length - startIndex;
                long[] arrRange = new long[length];
                Array.Copy(idList, startIndex, arrRange, 0, length);
                int affectedNum = 0;
                List<MyPostParameter> pars = new List<MyPostParameter>();
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(arrRange)));
                if(null!=status)
                    pars.Add(new MyPostParameter("status", status));
                if (price.HasValue)
                    pars.Add(new MyPostParameter("price", price));
                if (null!=url)
                    pars.Add(new MyPostParameter("url", url));
                if (null != matchType)
                    pars.Add(new MyPostParameter("matchType", matchType));
                MyHttpRequest req = new MyHttpRequest(this);

                req.methodType = MethodType.updateByIdList;
                req.postResponse(pars);
                req.GetRes(ref res);
                switch (req.status)
                {
                    case ResultStatus.Succeed:
                        AffectedRecords apiRes = Json.Deserialize<AffectedRecords>(req.data);
                        if (null == apiRes)
                            break;
                        res.Failures = apiRes.failures;
                        affectedRecords = apiRes.affectedRecords;
                        break;
                    case ResultStatus.Fail:
                        break;
                    case ResultStatus.DataIsEmpty:
                        break;
                }
                affectedRecords += affectedNum;
            }
            return res;
        }

        /// <summary>
        /// 删除一个关键词
        /// </summary>
        /// <param name="id">关键词id</param>
        /// <param name="affectedRecords">影响函数</param>
        /// <returns>Result.</returns>
        public Result deleteById(long id, ref int affectedRecords)
        {
            return deleteByIdList(new long[] { id }, ref affectedRecords);
        }

        /// <summary>
        /// 批量删除关键词
        /// </summary>
        /// <param name="idList">json格式的关键词id数组</param>
        /// <param name="affectedRecords">影响的行数</param>
        /// <returns>Result.</returns>
        public Result deleteByIdList(long[] idList, ref int affectedRecords)
        {
            if (idList == null)
                idList = new long[0];
            Result res = new Result();
            affectedRecords = 0;
            int maxRequestCount = MyConfig.MaxRequestCount;
            int count = idList.Length / maxRequestCount;
            if (idList.Length % maxRequestCount != 0)
                count += 1;
            for (int i = 0; i < count; i++)
            {
                int startIndex = i * maxRequestCount;
                int endIndex = maxRequestCount * (i + 1) - 1;
                int length = maxRequestCount;
                if (endIndex > idList.Length - 1)
                    length = idList.Length - startIndex;
                long[] arrRange = new long[length];
                Array.Copy(idList, startIndex, arrRange, 0, length);
                int affectedNum = 0;

                List<MyPostParameter> pars = new List<MyPostParameter>();
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(arrRange)));
                MyHttpRequest req = new MyHttpRequest(this);
                req.methodType = MethodType.deleteByIdList;
                req.postResponse(pars);
                req.GetRes(ref res);
                switch (req.status)
                {
                    case ResultStatus.Succeed:
                        AffectedRecords apiRes = Json.Deserialize<AffectedRecords>(req.data);
                        if (null == apiRes)
                            break;
                        res.Failures = apiRes.failures;
                        affectedNum = apiRes.affectedRecords;
                        break;
                    case ResultStatus.Fail:
                        break;
                    case ResultStatus.DataIsEmpty:
                        break;
                }
                affectedRecords += affectedNum;
            }
            return res;
        }

        /// <summary>
        /// 根据关键词id 获取关键词所有信息
        /// </summary>
        /// <param name="id">关键词id</param>
        /// <param name="keyword">关键词实体</param>
        /// <returns>Result.</returns>
        public Result getInfoById(long id, ref QH360MKeyword keyword)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("id", id));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.getInfoById;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    QH360MKeyword apiRes = Json.Deserialize<QH360MKeyword>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    keyword = apiRes;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
        /// <summary>
        /// 据关键词id 获取关键词所有信息
        /// </summary>
        /// <param name="idList">json格式的关键词id数组.</param>
        /// <param name="liKeywords">关键词实体列表.</param>
        /// <returns>Result.</returns>
        public Result getInfoByIdList(long[] idList, ref List<QH360MKeyword> liKeywords)
        {
            if (idList == null)
                idList = new long[0];
            List<MyPostParameter> pars = new List<MyPostParameter>();
            pars.Add(new MyPostParameter("idList", Json.SerializeObject(idList)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);

            req.methodType = MethodType.getInfoByIdList;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MKeywordList apiRes = Json.Deserialize<MKeywordList>(req.data);
                    if (null == apiRes)
                        break;
                    res.Failures = apiRes.failures;
                    liKeywords = apiRes.keywordList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }


        /// <summary>
        /// 批量获取关键词状态 
        /// </summary>
        /// <param name="idList">要获取状态的关键词id</param>
        /// <param name="liKeywords">返回状态model</param>
        /// <returns></returns>
        public Result getStatusByIdList(long[] idList, ref List<QH360MKeyword> liKeywords)
        {
            if (idList == null)
                idList = new long[0];
            Result res = new Result();
            liKeywords = new List<QH360MKeyword>();
            int maxRequestCount = MyConfig.MaxRequestCount;
            int count = idList.Length / maxRequestCount;
            if (idList.Length % maxRequestCount != 0)
                count += 1;
            for (int i = 0; i < count; i++)
            {
                int startIndex = i * maxRequestCount;
                int endIndex = maxRequestCount * (i + 1) - 1;
                int length = maxRequestCount;
                if (endIndex > idList.Length - 1)
                    length = idList.Length - startIndex;
                long[] arrRange = new long[length];
                Array.Copy(idList, startIndex, arrRange, 0, length);
                List<MyPostParameter> pars = new List<MyPostParameter>();
                pars.Add(new MyPostParameter("idList", Json.SerializeObject(arrRange)));
                MyHttpRequest req = new MyHttpRequest(this);
                req.methodType = MethodType.getStatusByIdList;
                req.postResponse(pars);
                req.GetRes(ref res);
                switch (req.status)
                {
                    case ResultStatus.Succeed:
                        MKeywordList apiRes = Json.Deserialize<MKeywordList>(req.data);
                        if (null == apiRes)
                            break;
                        res.Failures = apiRes.failures;
                        if (apiRes.keywordList != null)
                            liKeywords.AddRange(apiRes.keywordList);
                        break;
                    case ResultStatus.Fail:
                        break;
                    case ResultStatus.DataIsEmpty:
                        break;
                }
            }
            return res;
        }
        /// <summary>
        /// 询某个开始时间至当前时间所有被编辑过（新增、修改、删除）的关键词 id列表
        /// </summary>
        /// <param name="fromTime">起始时间</param>
        /// <param name="idList">返回的id列表</param>
        /// <returns></returns>
        public Result getChangedIdList(string fromTime, ref long[] idList)
        {
            List<MyPostParameter> pars = new List<MyPostParameter>();
            if (!string.IsNullOrEmpty(fromTime))
                pars.Add(new MyPostParameter("fromTime", string.Format(new CustomDateFormatter(), "{0:L}", fromTime)));
            Result res = new Result();
            MyHttpRequest req = new MyHttpRequest(this);
            req.methodType = MethodType.getChangedIdList;
            req.postResponse(pars);
            req.GetRes(ref res);
            switch (req.status)
            {
                case ResultStatus.Succeed:
                    MKeywordIdList m = Json.Deserialize<MKeywordIdList>(req.data);
                    if (null == m)
                        break;
                    res.Failures = m.failures;
                    idList = m.keywordIdList;
                    break;
                case ResultStatus.Fail:
                    break;
                case ResultStatus.DataIsEmpty:
                    break;
            }
            return res;
        }
        #endregion

        #region Json解析辅助类
        /// <summary>
        /// Creative Id 列表
        /// </summary>
        [Serializable]
        internal class MKeywordIdList : ApiResult
        {
            /// <summary>
            /// Gets or sets the keyword id list.
            /// </summary>
            /// <value>The keyword id list.</value>
            public long[] keywordIdList { get; set; }
        }
        /// <summary>
        /// Creative Item 对象
        /// </summary>
        [Serializable]
        internal class MKeywordIdItem : ApiResult
        {
            /// <summary>
            /// Gets or sets the id.
            /// </summary>
            /// <value>The id.</value>
            public long id { get; set; }
        }

        /// <summary>
        /// Keyword Id 列表
        /// </summary>
        [Serializable]
        internal class MKeywordList : ApiResult
        {
            /// <summary>
            /// Gets or sets the keyword list.
            /// </summary>
            /// <value>The keyword list.</value>
            public List<QH360MKeyword> keywordList { get; set; }
        }

        #endregion
        
    }
}
